/*
* All questions regarding the software should be addressed to
* 
*       Prof. Charles A. Bouman
*       Purdue University
*       School of Electrical and Computer Engineering
*       1285 Electrical Engineering Building
*       West Lafayette, IN 47907-1285
*       USA
*       +1 765 494 0340
*       +1 765 494 3358 (fax)
*       email:  bouman@ecn.purdue.edu
*       http://www.ece.purdue.edu/~bouman
* 
* Copyright (c) 1995 The Board of Trustees of Purdue University.
*
* Permission to use, copy, modify, and distribute this software and its
* documentation for any purpose, without fee, and without written agreement is
* hereby granted, provided that the above copyright notice and the following
* two paragraphs appear in all copies of this software.
*
* IN NO EVENT SHALL PURDUE UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF PURDUE UNIVERSITY HAS
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* PURDUE UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
* AND PURDUE UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
* UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/


/* clust.c */

#include <math.h>

#include "clust.h"
#include "clustermessage.h"

static void error(char *name);

main(argc, argv) 
int  argc;
char *argv[];
{
    FILE *fp,*info_fp;

    int i,j,k;
    int init_num_of_subclasses,max_num;
    int vector_dimension;
    int nclasses;
    int num_of_samples;
    char option[16];
    char fname[512];
    struct SigSet S;
    struct ClassSig *Sig;

    clusterMessageInit(argv[0]);
    clusterMessageVerboseLevel = 1;
    if((argc!=4) && (argc!=5)) {
      clusterError( 1,
          "\n\nUsage: %s #_subclasses info_file output_params [option]\n\n"    
          "    #_subclasses - initial number of clusters for each class\n\n"
          "    info_file - name of file which contains the following information:\n"
          "      <# of classes>\n"
          "      <data vector length>\n"
          "      <class 1 data file name> <# of data vectors in class 1>\n"
          "      <class 2 data file name> <# of data vectors in class 2>\n"
          "                     .                        .\n"
          "                     .                        .\n"
          "                     .                        .\n"
          "      <last class data file name> <# of data vectors in last class>\n\n"
          "    output_params - name of file containing output cluster"
          " parameters\n\n"
          "    option - (optional) controls clustering model\n"
          "      full - (default) use full convariance matrices\n"
          "      diag - use diagonal convariance matrices",
          argv[0] );
           
    } else if(argc==4) {
      /* set default option */
      sprintf(option,"full") ;

    } else {
      sscanf(argv[4], "%s", option) ;
      if((strcmp(option, "full")!=0) && (strcmp(option, "diag")!=0)) {
        clusterError( 1,
            "\nInvalid option: %s\n\n"    
            "There are 2 valid assumptions:\n"
            "    full - default option which allows full convariance matrices\n"
            "    diag - use diagonal convariance matrices\n\n",
            option);
      }
    }
           

    /* read number of initial subclasses to use */
    sscanf(argv[1],"%d",&init_num_of_subclasses);
    
    /* open information file */
    if((info_fp=fopen(argv[2],"r"))==NULL) {
      clusterError(1, "can't open information file");
    }

    /* read number of classes from info file */
    fscanf(info_fp,"%d\n",&nclasses);

    /* read vector dimension from info file */
    fscanf(info_fp,"%d\n",&vector_dimension);


    /* Initialize SigSet data structure */
    I_InitSigSet (&S);
    I_SigSetNBands (&S, vector_dimension);
    I_SetSigTitle (&S, "test signature set");


    /* Allocate memory for cluster signatures */
    for(k=0; k<nclasses; k++) {
      Sig = I_NewClassSig(&S);
      I_SetClassTitle (Sig, "test class signature");
      for(i=0; i<init_num_of_subclasses; i++)
        I_NewSubSig (&S, Sig);
    }

    /* Perform clustering for each class */
    for(k=0; k<nclasses; k++) {
      /* read class k data file name */
      fscanf(info_fp,"%s",fname);

      /* read number of samples for class k */
      fscanf(info_fp,"%d\n",&num_of_samples);

      Sig = &(S.ClassSig[k]);

      I_AllocClassData (&S, Sig, num_of_samples);

      /* Read Data */
      if((fp=fopen(fname,"r"))==NULL) {
        clusterError(1, "can't open data file %s", fname);
      }

      for(i=0; i<Sig->ClassData.npixels; i++) {
        for(j=0; j<vector_dimension; j++) {
          fscanf(fp,"%lf",&(Sig->ClassData.x[i][j]) );
        }
        fscanf(fp,"\n");
      }
      fclose(fp);

      if(1<=clusterMessageVerboseLevel) {
        clusterInfo("Start clustering class %d",k);
      }
        
      if(strcmp(option, "diag")==0) {
        /* assume covariance matrices to be diagonal */
        subcluster2(&S,k,(int) 0,&max_num);
          
      } else {
        /* no assumption for covariance matrices */
        subcluster(&S,k,(int) 0,&max_num); 
      }
      
      if(2<=clusterMessageVerboseLevel) {
        clusterInfo("Maximum number of subclasses = %d",max_num);
      }

      I_DeallocClassData(&S, Sig);

      /* Write out intermediate results to output parameter file */
      if((fp=fopen(argv[3],"w"))==NULL) {
          clusterError(1, "can't open parameter output file");
      }
      I_WriteSigSet(fp, &S);
      fclose(fp);
    }
    fclose(info_fp);

    /* De-allocate cluster signature memory */
    I_TermSigSet (&S); 
}
